
 DROP SPECIFIC PROCEDURE DELETE_XACTIONS
@

call sysproc.set_routine_opts('ISOLATION UR') @

CREATE PROCEDURE DELETE_XACTIONS (
   IN i_tabname CHAR(50),
   IN i_fraud_resultid INTEGER,     -- added to prevent hard-coding (S0411604)
   IN i_commit_point INTEGER,
   IN i_start_date TIMESTAMP 
   )       
SPECIFIC DELETE_XACTIONS
DYNAMIC RESULT SETS 1               -- Now the proc returns a Result set (S0405876)
LANGUAGE SQL
-----------------------------------------------------------------------------------------------------------------------
-- File:  DELETE_XACTIONS.SQL
--
--
-- Description:
--     Delete NON FRAUD related transactions from the system of record database.
--
--     A transaction WILL NOT be deleted if the account number
--     exists in the alert table with a status of ACTIVE or CHECKOUT
--     even if the other criteria are true.
--     Other than that, look out!
--
--     Commit every i_commit_point rows to control the redo generation.
--
--
-- Called By:
--
-- Calls:
--     N/A
--
-- Notes:
--
--
-- Usage:
--      db2 "call delete_xactions('FCM_AUTH', 7, 10000, '2000-06-15-10.50.40.000000')"
--
-- History:
--     Date          User                Notes
--     2006/11/20    jicimi              created
--     2006/12/19    sasgxw              modified to handle transactions
--                                       using rqo_proc_utc_datetime
--     2007/01/04    dofran              removed date function from rqo_proc_utc_datetime to insure index would be used,
--                                       changed in parm i_start_date from date to timestamp
--     2007/02/13    sinpad              Modified to allow PROC SQL invocation of procedure and to return a result set.
--                                       Also, changed EXIT handler to CONTINUE handler (DEFECT S0405876)
--     2007/03/13    sinpad              Added IN parameter for Fraud Result ID and modified logic for deleting 
--                                       Non-Fraud transactions so that alert status is not checked anymore
--                                       (DEFECT S0411604).
--     2007/05/03    jicimi              changed subselect 
--                                       removed for update clause on select  
--                                       added explicit call to set isolation leve to ur
--     2007/05/30    jicimi              removed is null predicate from subselect on initial select
--                                       alerts that have not been touched by an analyst are purge eligible
--     2007/10/19    jicimi              initial changes for R2
--     2007/12/21    sinpbh		 Logic of delete_basedon is added.
--					 Now we have only one procedure to purge transactiont tables and SMON table.
--     2008/07/07    leushe              add the 2 new debit transaction tables FCM_DDAC, FCM_DANC to the valid tables list  
--     18DEC2008     LEUSHE              S0553598 Removed checking on the SQLSTATE variable. Changed this by setting 
--                                       the v_sqlstate variable SQLSTATE, the conditionally checking v_sqlstate.
--                                       In db2 9.5, accessing the SQLSTATE variable, resets it. Thus, the IF would reset the SQLSTATE 
--                                       value. 
---    06/15/2009    dofran              S0530934: changed account number field from rqo_acct_num to xqo_cust_num.
--                                       Criteria changed from account number to customer number. Added FCM_SMON table.   
--     2010/10/22    leushe              Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--     2010/12/09    leushe              added new txn types CSAB and CSAE
--     2010/12/22    leushe              fixed ambigous reference to xqo_Cust_num
--     2011/01/14    leushe              Removed logic around FCM_SMON deletes, since the FCM_SMON table has been removed
--     2011/01/27    leushe  			 changed the preservation of CF txn logic to determine which "alert type" fields
--                          			 are on the txn table, and compare those columns with the alert_value of the alert
--                          			 by alert_type 
-----------------------------------------------------------------------------------------------------------------------
P1: BEGIN
    /* Added logic of delete_basedon */
    DECLARE l_del_count_smon INTEGER DEFAULT 0;
    DECLARE l_rows_deleted_smon INTEGER DEFAULT 0;
    DECLARE l_id_smon BIGINT;    
    /* End */
    DECLARE l_del_count INTEGER DEFAULT 0;
    DECLARE l_counter INTEGER DEFAULT 0;
    DECLARE l_rows_deleted INTEGER DEFAULT 0;
    DECLARE l_date      DATE   ;
    DECLARE l_id BIGINT;
    DECLARE v_alert_type CHAR(1);
    DECLARE v_fieldname VARCHAR(50);
        /* Added logic of delete_basedon */
    DECLARE l_stmtTemplate_smon varchar(1000);
    DECLARE l_delStmt_smon varchar(1000);
    /* End */

    DECLARE l_stmtTemplate varchar(10000);
    DECLARE l_prepStmtTemplate varchar(10000) DEFAULT '';
    DECLARE at_end SMALLINT DEFAULT 0;    
    DECLARE l_delStmt varchar(1000);
    DECLARE l_problem INTEGER DEFAULT 0;
    DECLARE v_sqlstate CHAR(5);     -- variable for SQLSTATE
    DECLARE v_sqlcode INTEGER;      -- variable for SQLCODE
    DECLARE v_delete_count INTEGER;      -- variable for deleted row count
    /* Added logic of delete_basedon */
    DECLARE v_delete_count_smon INTEGER;      -- variable for deleted row count
    /* End */
    
    DECLARE not_found CONDITION FOR SQLSTATE '02000';
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    
    DECLARE l_stmt STATEMENT; 

    /* Added logic of delete_basedon */
    DECLARE l_stmt_smon STATEMENT;
    /* End */
    DECLARE c_stmt CURSOR WITH HOLD FOR l_stmt;
    /* Added logic of delete_basedon */
    DECLARE c_stmt_smon CURSOR WITH HOLD FOR l_stmt_smon;
    /* End */    
--'( alrt.alert_Value = txn.'||rtrim(field_name)||'  and alrt.alert_type = ''||alert_Type||'')'
    DECLARE c_prep CURSOR FOR
	  select distinct field_name,alert_type
	    from fcm_alert_type at, fcm_field_definition fd, fcm_transaction_Definition td,
	    FCM_transaction_component tc, fcm_component_segment cs,fcm_segment_Definition sd,fcm_component_value cv
	  where fd.field_id = at.KEY_FIELD_ID
          and sd.segment_id = fd.SEGMENT_ID
          and cs.SEGMENT_ID = sd.SEGMENT_ID
          and tc.TRANSACTION_ID = td.TRANSACTION_ID
          and tc.COMPONENT_VALUE_ID = cv.COMPONENT_VALUE_ID
          and cs.COMPONENT_VALUE_ID = cv.COMPONENT_VALUE_ID
          and td.SOR_TABLE_NAME = i_tabname
          and fd.field_name <> 'xqo_cust_num';    
    
    /* Declare cursor to return result set to calling program - S0405876 */    
    DECLARE c_result CURSOR WITH RETURN FOR
        SELECT i_tabname as TARGET_TAB
              ,v_delete_count as DELETE_ROWCOUNT
              ,v_sqlstate as OUT_SQLSTATE
              ,v_sqlcode as OUT_SQLCODE
        FROM SYSIBM.SYSDUMMY1;
    
    /* Changed EXIT handler to CONTINUE handler to always execute the OPEN cursor statement - S0405876 */
    DECLARE EXIT HANDLER FOR SQLEXCEPTION
       SELECT SQLSTATE,
              SQLCODE
       INTO   v_sqlstate,
              v_sqlcode
       FROM SYSIBM.SYSDUMMY1;
       
	DECLARE CONTINUE HANDLER for not_found
      SET at_end = 1;    
  
  /* Initialize variables */
  VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;       


    --
    -- cursor/query to select the canidate rows for deletion
    --
    -- S0530934: --
    SET l_stmtTemplate =
         'select cmx_tran_id from '|| rtrim(i_tabname) || ' as txn where ( rqo_proc_utc_datetime <= ? and not exists ( select 1 from fcm_alert as alrt where result_id = ? and ('; 
  

		
    --
    -- cursor to delete rows from previous statement
    --
    SET l_delStmt = 'DELETE FROM '|| rtrim(i_tabname) || ' WHERE CURRENT OF c_stmt ';

 
    IF (exists (select 1 from fcm_transaction_Definition where sor_Table_name = i_tabname) )
    	THEN
        open c_prep ;

          -- keep searching for an alert until the strategy is depleted or no more strategies
          while ( at_end = 0 ) do
            FETCH c_prep into v_fieldname,v_alert_type;
            if (l_counter>0 AND at_end = 0) THEN
              set l_prepStmtTemplate = l_prepStmtTemplate||' OR '||'( alrt.alert_Value = txn.'||rtrim(v_fieldname)||'  and alrt.alert_type = '''||v_alert_type||''')';
            ELSE     
              IF ( at_end=0 ) THEN
                set l_prepStmtTemplate = l_prepStmtTemplate||'( alrt.alert_Value = txn.'||rtrim(v_fieldname)||'  and alrt.alert_type = '''||v_alert_type||''')';
              ELSE
                  SET v_sqlcode = -1;
                  SET v_sqlstate = '-1';
                  SET v_delete_count = 0;
--                  OPEN c_result;                       
--                  RETURN -1;
              END IF;
            END IF; 
            set l_counter=l_counter+1;
        
          end while;
            
          close c_prep;

            
          set l_stmtTemplate = l_stmtTemplate||l_prepStmtTemplate||')))';
          set at_end=0;
        
          
          PREPARE l_stmt FROM l_stmtTemplate;
          OPEN c_stmt USING i_start_date, i_fraud_resultid;

          VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
          WHILE ( v_sqlstate = '00000' ) DO

            FETCH c_stmt INTO l_id ;
            VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
            IF ( v_sqlstate = '00000' and at_end=0 ) THEN       
              EXECUTE IMMEDIATE l_delStmt;

              SET l_del_count = l_del_count + 1;
                IF ( l_del_count = i_commit_point ) THEN
                  COMMIT;
                  SET l_rows_deleted = l_rows_deleted + l_del_count;
                  SET l_del_count = 0;
                END IF;
              ELSE 
                  COMMIT;
                  SET v_sqlstate = '0001';
            END IF;

          END WHILE;

          IF ( l_del_count > 0 ) THEN
            COMMIT;
            SET l_rows_deleted = l_rows_deleted + l_del_count;
          END IF;

          CLOSE c_stmt;
          
          VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
          
          SET v_delete_count = l_rows_deleted;
      
       ELSE IF ( i_tabname = 'FCM_SMON') THEN
           /* Added logic of delete_basedon */   
    	--
        -- cursor/query to select the canidate rows for deletion
        --
   
       SET l_stmtTemplate_smon = ' select cmx_tran_id from '|| rtrim( i_tabname) || ' where cmx_create_timestamp <= ?    FOR UPDATE' ;
    
        --
        -- cursor to delete rows from previous statement
        --
        SET l_delStmt_smon = 'DELETE FROM '|| rtrim(i_tabname) || ' WHERE CURRENT OF c_stmt_smon ';
    
    /* End */
           
    /* Added logic of delete_basedon */
                 
             PREPARE l_stmt_smon FROM l_stmtTemplate_smon;
                 OPEN c_stmt_smon USING i_start_date;
             
                 SET v_sqlstate = SQLSTATE;
                
                 WHILE ( v_sqlstate = '00000' ) DO
    
                    FETCH c_stmt_smon INTO l_id_smon ;
                    SET v_sqlstate = SQLSTATE;
                    IF ( v_sqlstate = '00000' and at_end=0 ) THEN
                       EXECUTE IMMEDIATE l_delStmt_smon;
             
                       SET l_del_count_smon = l_del_count_smon + 1;
                       IF ( l_del_count_smon = i_commit_point ) THEN
                          COMMIT;
                          SET l_rows_deleted_smon = l_rows_deleted_smon + l_del_count_smon;
                          SET l_del_count_smon = 0;
                       END IF;
                    ELSE 
                        COMMIT;
                        SET v_sqlstate = '0001';
                    END IF;
             
                 END WHILE;
             
             
                 IF ( l_del_count_smon > 0 ) THEN
                    COMMIT;
                    SET l_rows_deleted_smon = l_rows_deleted_smon + l_del_count_smon;
                 END IF;
             
                 CLOSE c_stmt_smon;
             
                 
                 
            	VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
             SET v_delete_count = l_rows_deleted_smon;
     /* End */

    ELSE
      SET v_sqlcode = -1;
      SET v_sqlstate = '-1';
      SET v_delete_count = 0;
    END IF;
         END IF;
    
    /* Open the cursor to return result set - S0405876 */
    OPEN c_result;

  END P1@




GRANT EXECUTE ON SPECIFIC PROCEDURE DELETE_XACTIONS TO ROLE BATCHROLE@